@@ -23,7 +23,10 @@ import scala.collection.immutable.Map$;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.nio.ByteBuffer;
+import java.nio.ByteOrder;
 import java.util.Arrays;
+import java.util.UUID;
 
 /**
  * This class defines constants related to the variant format and provides functions for
@@ -121,6 +124,9 @@ public class VariantUtil {
   // string size) + (size bytes of string content).
   public static final int LONG_STR = 16;
 
+  // UUID, 16-byte big-endian.
+  public static final int UUID = 20;
+
   public static final byte VERSION = 1;
   // The lower 4 bits of the first metadata byte contain the version.
   public static final byte VERSION_MASK = 0x0F;
@@ -131,8 +137,10 @@ public class VariantUtil {
   public static final int U24_SIZE = 3;
   public static final int U32_SIZE = 4;
 
-  // Both variant value and variant metadata need to be no longer than 16MiB.
-  public static final int SIZE_LIMIT = U24_MAX + 1;
+  // Both variant value and variant metadata need to be no longer than 128MiB.
+  // Note: to make tests more reliable, we set the max size to 16Mib to avoid OOM in tests.
+  public static final int SIZE_LIMIT =
+    System.getenv("SPARK_TESTING") != null ? U24_MAX + 1 : 128 * 1024 * 1024;
 
   public static final int MAX_DECIMAL4_PRECISION = 9;
   public static final int MAX_DECIMAL8_PRECISION = 18;
@@ -171,6 +179,12 @@ public class VariantUtil {
         Map$.MODULE$.<String, String>empty(), null, new QueryContext[]{}, "");
   }
 
+  static SparkRuntimeException unknownPrimitiveTypeInVariant(int id) {
+    return new SparkRuntimeException("UNKNOWN_PRIMITIVE_TYPE_IN_VARIANT",
+            new scala.collection.immutable.Map.Map1<>("id", Integer.toString(id)), null,
+            new QueryContext[]{}, "");
+  }
+
   // An exception indicating that an external caller tried to call the Variant constructor with
   // value or metadata exceeding the 16MiB size limit. We will never construct a Variant this large,
   // so it should only be possible to encounter this exception when reading a Variant produced by
@@ -204,7 +218,7 @@ public class VariantUtil {
 
   // Read a little-endian unsigned int value from `bytes[pos, pos + numBytes)`. The value must fit
   // into a non-negative int (`[0, Integer.MAX_VALUE]`).
-  static int readUnsigned(byte[] bytes, int pos, int numBytes) {
+  public static int readUnsigned(byte[] bytes, int pos, int numBytes) {
     checkIndex(pos, bytes.length);
     checkIndex(pos + numBytes - 1, bytes.length);
     int result = 0;
@@ -233,6 +247,12 @@ public class VariantUtil {
     TIMESTAMP_NTZ,
     FLOAT,
     BINARY,
+    UUID,
+  }
+
+  public static int getTypeInfo(byte[] value, int pos) {
+    checkIndex(pos, value.length);
+    return (value[pos] >> BASIC_TYPE_BITS) & TYPE_INFO_MASK;
   }
 
   // Get the value type of variant value `value[pos...]`. It is only legal to call `get*` if
@@ -280,8 +300,10 @@ public class VariantUtil {
             return Type.BINARY;
           case LONG_STR:
             return Type.STRING;
+          case UUID:
+            return Type.UUID;
           default:
-            throw malformedVariant();
+            throw unknownPrimitiveTypeInVariant(typeInfo);
         }
     }
   }
@@ -331,8 +353,10 @@ public class VariantUtil {
           case BINARY:
           case LONG_STR:
             return 1 + U32_SIZE + readUnsigned(value, pos + 1, U32_SIZE);
+          case UUID:
+            return 17;
           default:
-            throw malformedVariant();
+            throw unknownPrimitiveTypeInVariant(typeInfo);
         }
     }
   }
@@ -356,8 +380,9 @@ public class VariantUtil {
   // Get a long value from variant value `value[pos...]`.
   // It is only legal to call it if `getType` returns one of `Type.LONG/DATE/TIMESTAMP/
   // TIMESTAMP_NTZ`. If the type is `DATE`, the return value is guaranteed to fit into an int and
-  // represents the number of days from the Unix epoch. If the type is `TIMESTAMP/TIMESTAMP_NTZ`,
-  // the return value represents the number of microseconds from the Unix epoch.
+  // represents the number of days from the Unix epoch.
+  // If the type is `TIMESTAMP/TIMESTAMP_NTZ`, the return value represents the number of
+  // microseconds from the Unix epoch.
   // Throw `MALFORMED_VARIANT` if the variant is malformed.
   public static long getLong(byte[] value, int pos) {
     checkIndex(pos, value.length);
@@ -401,7 +426,7 @@ public class VariantUtil {
 
   // Get a decimal value from variant value `value[pos...]`.
   // Throw `MALFORMED_VARIANT` if the variant is malformed.
-  public static BigDecimal getDecimal(byte[] value, int pos) {
+  public static BigDecimal getDecimalWithOriginalScale(byte[] value, int pos) {
     checkIndex(pos, value.length);
     int basicType = value[pos] & BASIC_TYPE_MASK;
     int typeInfo = (value[pos] >> BASIC_TYPE_BITS) & TYPE_INFO_MASK;
@@ -433,7 +458,11 @@ public class VariantUtil {
       default:
         throw unexpectedType(Type.DECIMAL);
     }
-    return result.stripTrailingZeros();
+    return result;
+  }
+
+  public static BigDecimal getDecimal(byte[] value, int pos) {
+    return getDecimalWithOriginalScale(value, pos).stripTrailingZeros();
   }
 
   // Get a float value from variant value `value[pos...]`.
@@ -481,6 +510,20 @@ public class VariantUtil {
     throw unexpectedType(Type.STRING);
   }
 
+  // Get a UUID value from variant value `value[pos...]`.
+  // Throw `MALFORMED_VARIANT` if the variant is malformed.
+  public static UUID getUuid(byte[] value, int pos) {
+    checkIndex(pos, value.length);
+    int basicType = value[pos] & BASIC_TYPE_MASK;
+    int typeInfo = (value[pos] >> BASIC_TYPE_BITS) & TYPE_INFO_MASK;
+    if (basicType != PRIMITIVE || typeInfo != UUID) throw unexpectedType(Type.UUID);
+    int start = pos + 1;
+    checkIndex(start + 15, value.length);
+    // UUID values are big-endian, so we can't use VariantUtil.readLong().
+    ByteBuffer bb = ByteBuffer.wrap(value, start, 16).order(ByteOrder.BIG_ENDIAN);
+    return new UUID(bb.getLong(), bb.getLong());
+  }
+
   public interface ObjectHandler<T> {
     /**
      * @param size Number of object fields.
