@@ -453,28 +453,29 @@ class StreamingDeduplicationSuite extends StateStoreMetricsTest {
     }
   }
 
-  test("SPARK-39650: recovery from checkpoint having all columns as value schema") {
-    // NOTE: We are also changing the schema of input compared to the checkpoint. In the checkpoint
-    // we define the input schema as (String, Int).
-    val inputData = MemoryStream[(String, Int, String)]
-    val dedupe = inputData.toDS().dropDuplicates("_1")
+  Seq("3.3.0", "3.5.1").foreach { sparkVersion =>
+    test("SPARK-39650: recovery from checkpoint having all columns as value schema " +
+      s"with sparkVersion=$sparkVersion") {
+      // NOTE: We are also changing the schema of input compared to the checkpoint.
+      // In the checkpoint we define the input schema as (String, Int).
+      val inputData = MemoryStream[(String, Int, String)]
+      val dedupe = inputData.toDS().dropDuplicates("_1")
 
-    // The fix will land after Spark 3.3.0, hence we can check backward compatibility with
-    // checkpoint being built from Spark 3.3.0.
-    val resourceUri = this.getClass.getResource(
-      "/structured-streaming/checkpoint-version-3.3.0-streaming-deduplication/").toURI
+      val resourcePath = "/structured-streaming/checkpoint-version-" + sparkVersion +
+        "-streaming-deduplication/"
+      val resourceUri = this.getClass.getResource(resourcePath).toURI
 
-    val checkpointDir = Utils.createTempDir().getCanonicalFile
-    // Copy the checkpoint to a temp dir to prevent changes to the original.
-    // Not doing this will lead to the test passing on the first run, but fail subsequent runs.
-    FileUtils.copyDirectory(new File(resourceUri), checkpointDir)
+      val checkpointDir = Utils.createTempDir().getCanonicalFile
+      // Copy the checkpoint to a temp dir to prevent changes to the original.
+      // Not doing this will lead to the test passing on the first run, but fail subsequent runs.
+      FileUtils.copyDirectory(new File(resourceUri), checkpointDir)
 
-    inputData.addData(("a", 1, "dummy"))
-    inputData.addData(("a", 2, "dummy"), ("b", 3, "dummy"))
+      inputData.addData(("a", 1, "dummy"))
+      inputData.addData(("a", 2, "dummy"), ("b", 3, "dummy"))
 
-    testStream(dedupe, Append)(
-      StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
-      /*
+      testStream(dedupe, Append)(
+        StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
+        /*
         Note: The checkpoint was generated using the following input in Spark version 3.3.0
         AddData(inputData, ("a", 1)),
         CheckLastBatch(("a", 1)),
@@ -482,16 +483,57 @@ class StreamingDeduplicationSuite extends StateStoreMetricsTest {
         CheckLastBatch(("b", 3))
        */
 
-      AddData(inputData, ("a", 5, "a"), ("b", 2, "b"), ("c", 9, "c")),
-      CheckLastBatch(("c", 9, "c"))
-    )
+        AddData(inputData, ("a", 5, "a"), ("b", 2, "b"), ("c", 9, "c")),
+        CheckLastBatch(("c", 9, "c"))
+      )
+    }
+  }
+
+  Seq("3.3.0", "3.5.1").foreach { sparkVersion =>
+    test("SPARK-39650: recovery from checkpoint with changes on key schema " +
+      s"are not allowed with sparkVersion=$sparkVersion") {
+      // NOTE: We are also changing the schema of input compared to the checkpoint.
+      // In the checkpoint we define the input schema as (String, Int).
+      val inputData = MemoryStream[(String, Int, String)]
+      val dedupe = inputData.toDS().dropDuplicates("_1", "_2")
+
+      val resourcePath = "/structured-streaming/checkpoint-version-" + sparkVersion +
+        "-streaming-deduplication/"
+      val resourceUri = this.getClass.getResource(resourcePath).toURI
+
+      val checkpointDir = Utils.createTempDir().getCanonicalFile
+      // Copy the checkpoint to a temp dir to prevent changes to the original.
+      // Not doing this will lead to the test passing on the first run, but fail subsequent runs.
+      FileUtils.copyDirectory(new File(resourceUri), checkpointDir)
+
+      inputData.addData(("a", 1, "dummy"))
+      inputData.addData(("a", 2, "dummy"), ("b", 3, "dummy"))
+
+      // trying to evolve the key schema is not allowed and should throw an exception
+      val ex = intercept[StreamingQueryException] {
+        testStream(dedupe, Append)(
+          StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
+          AddData(inputData, ("a", 5, "a"), ("b", 2, "b"), ("c", 9, "c")),
+          CheckLastBatch(("a", 5, "a"), ("b", 2, "b"), ("c", 9, "c"))
+        )
+      }
+
+      // verify that the key schema not compatible error is thrown
+      checkError(
+        ex.getCause.asInstanceOf[SparkUnsupportedOperationException],
+        condition = "STATE_STORE_KEY_SCHEMA_NOT_COMPATIBLE",
+        parameters = Map("storedKeySchema" -> ".*",
+          "newKeySchema" -> ".*"),
+        matchPVals = true
+      )
+    }
   }
 
   test("collation aware deduplication") {
     val inputData = MemoryStream[(String, Int)]
     val result = inputData.toDF()
       .select(col("_1")
-        .try_cast(StringType("UNICODE")).as("str"),
+        .try_cast(StringType("UTF8_BINARY")).as("str"),
         col("_2").as("int"))
       .dropDuplicates("str")
 
@@ -514,7 +556,7 @@ class StreamingDeduplicationSuite extends StateStoreMetricsTest {
     val inputData = MemoryStream[(String)]
     val result = inputData.toDF()
       .select(col("value")
-        .try_cast(StringType("UTF8_BINARY_LCASE")).as("str"))
+        .try_cast(StringType("UTF8_LCASE")).as("str"))
       .dropDuplicates("str")
 
     val ex = intercept[StreamingQueryException] {
@@ -525,13 +567,28 @@ class StreamingDeduplicationSuite extends StateStoreMetricsTest {
 
     checkError(
       ex.getCause.asInstanceOf[SparkUnsupportedOperationException],
-      errorClass = "STATE_STORE_UNSUPPORTED_OPERATION_BINARY_INEQUALITY",
+      condition = "STATE_STORE_UNSUPPORTED_OPERATION_BINARY_INEQUALITY",
       parameters = Map(
-        "schema" -> ".+\"str\":\"spark.UTF8_BINARY_LCASE\".+"
+        "schema" -> ".+\"str\":\"spark.UTF8_LCASE\".+"
       ),
       matchPVals = true
     )
   }
+
+  test("test that avro encoding is not supported") {
+    val inputData = MemoryStream[String]
+    val result = inputData.toDS().dropDuplicates()
+
+    val ex = intercept[Exception] {
+      withSQLConf(SQLConf.STREAMING_STATE_STORE_ENCODING_FORMAT.key -> "avro") {
+        testStream(result, Append)(
+          AddData(inputData, "a"),
+          ProcessAllAvailable()
+        )
+      }
+    }
+    assert(ex.getMessage.contains("State store encoding format as avro is not supported"))
+  }
 }
 
 @SlowSQLTest
